/*
 * Copyright [2022] [https://www.baiduc.com]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.baiduc.com
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.baiduc.com
 */
package vip.xiaonuo.hr.modular.count.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.hr.modular.baseemployeeinfo.entity.vo.PerNum;
import vip.xiaonuo.hr.modular.baseemployeeinfo.param.EmployeeInfoEoucation;
import vip.xiaonuo.hr.modular.count.entity.HrCount;
import vip.xiaonuo.hr.modular.count.param.HrCountAddParam;
import vip.xiaonuo.hr.modular.count.param.HrCountEditParam;
import vip.xiaonuo.hr.modular.count.param.HrCountIdParam;
import vip.xiaonuo.hr.modular.count.param.HrCountPageParam;
import vip.xiaonuo.hr.modular.count.service.HrCountService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 组织控制器
 *
 * @author 余
 * @date  2024/12/16 11:09
 */
@Api(tags = "组织控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class HrCountController {

    @Resource
    private HrCountService hrCountService;

    @ApiOperationSupport(order = 1)
    @ApiOperation("获取组织分页")
    @GetMapping("/biz/count/page")
    public CommonResult<Page<HrCount>> page(HrCountPageParam hrCountPageParam) {
        return CommonResult.data(hrCountService.page(hrCountPageParam));
    }

    @ApiOperationSupport(order = 2)
    @ApiOperation("添加组织")
    @CommonLog("添加组织")
    @PostMapping("/biz/count/add")
    public CommonResult<String> add(@RequestBody @Valid HrCountAddParam hrCountAddParam) {
        hrCountService.add(hrCountAddParam);
        return CommonResult.ok();
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑组织")
    @CommonLog("编辑组织")
    @PostMapping("/biz/count/edit")
    public CommonResult<String> edit(@RequestBody @Valid HrCountEditParam hrCountEditParam) {
        hrCountService.edit(hrCountEditParam);
        return CommonResult.ok();
    }

    @ApiOperationSupport(order = 4)
    @ApiOperation("删除组织")
    @CommonLog("删除组织")
    @PostMapping("/biz/count/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<HrCountIdParam> hrCountIdParamList) {
        hrCountService.delete(hrCountIdParamList);
        return CommonResult.ok();
    }

    @ApiOperationSupport(order = 5)
    @ApiOperation("获取组织详情")
    @GetMapping("/biz/count/detail")
    public CommonResult<HrCount> detail(@Valid HrCountIdParam hrCountIdParam) {
        return CommonResult.data(hrCountService.detail(hrCountIdParam));
    }

//    @ApiOperation("员工")
//    @PostMapping("/hr/baseemployeeinfo/gongLingDis")
//    public CommonResult<List<PerNum>> gongLingDis(@Valid EmployeeInfoEoucation employeeInfoEoucation) {
////        return CommonResult.data(hrBaseEmployeeInfoService.gongLingDis(employeeInfoEoucation));
//    }
}
