/*
 * Copyright [2022] [https://www.baiduc.com]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.baiduc.com
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.baiduc.com
 */
package vip.xiaonuo.hr.modular.count.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.hr.modular.count.entity.HrCount;
import vip.xiaonuo.hr.modular.count.mapper.HrCountMapper;
import vip.xiaonuo.hr.modular.count.param.HrCountAddParam;
import vip.xiaonuo.hr.modular.count.param.HrCountEditParam;
import vip.xiaonuo.hr.modular.count.param.HrCountIdParam;
import vip.xiaonuo.hr.modular.count.param.HrCountPageParam;
import vip.xiaonuo.hr.modular.count.service.HrCountService;

import java.util.List;

/**
 * 组织Service接口实现类
 *
 * @author 余
 * @date  2024/12/16 11:09
 **/
@Service
public class HrCountServiceImpl extends ServiceImpl<HrCountMapper, HrCount> implements HrCountService {

    @Override
    public Page<HrCount> page(HrCountPageParam hrCountPageParam) {
        QueryWrapper<HrCount> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(hrCountPageParam.getSortField(), hrCountPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(hrCountPageParam.getSortOrder());
            queryWrapper.orderBy(true, hrCountPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(hrCountPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(HrCount::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(HrCountAddParam hrCountAddParam) {
        HrCount hrCount = BeanUtil.toBean(hrCountAddParam, HrCount.class);
        this.save(hrCount);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(HrCountEditParam hrCountEditParam) {
        HrCount hrCount = this.queryEntity(hrCountEditParam.getId());
        BeanUtil.copyProperties(hrCountEditParam, hrCount);
        this.updateById(hrCount);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<HrCountIdParam> hrCountIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(hrCountIdParamList, HrCountIdParam::getId));
    }

    @Override
    public HrCount detail(HrCountIdParam hrCountIdParam) {
        return this.queryEntity(hrCountIdParam.getId());
    }

    @Override
    public HrCount queryEntity(String id) {
        HrCount hrCount = this.getById(id);
        if(ObjectUtil.isEmpty(hrCount)) {
            throw new CommonException("组织不存在，id值为：{}", id);
        }
        return hrCount;
    }
}
